# !/usr/bin/env python
# coding=utf-8
import sys
reload(sys)
sys.setdefaultencoding('utf8')
import subprocess
import os
import tornado.httpserver
from tornado.httpclient import HTTPClient
import tornado.ioloop
import tornado.web
import time
import traceback
import hmac
import json
from hashlib import sha1
import logging
import ctypes
import signal
from M2Crypto import RSA
from tornado import gen, locks
import struct
import base64

FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
logging.basicConfig(format=FORMAT)
logger = logging.getLogger('web')
logger.setLevel(logging.INFO)


def packUint16(x):
    return struct.pack('<H', int(x))


def packUint32(x):
    return struct.pack('<I', int(x))


def packInt32(x):
    return struct.pack('<i', int(x))


def packString(string):
    return packUint16(len(string)) + string


def packMap(m):
    ret = packUint16(len(m.items()))
    for k, v in m.items():
        ret += packUint16(k) + packString(v)
    return ret


def unpackUint16(x):
    return struct.unpack('<H', x)


def unpackUint32(x):
    return struct.unpack('<I', x)


def generateSignaure(staticKey, signKey, channelName, unixTs, randomInt):
    key = "\x00" * (32 - len(staticKey)) + staticKey
    content = key +\
        '{:0>10}'.format(unixTs) + \
        "%.8x" % (int(randomInt)) + \
        str(channelName)
    signature = hmac.new(signKey, content, sha1).hexdigest()
    return signature


def generateDynamicKey(staticKey, signKey, channelName, unixTs, randomInt):
    signature = generateSignaure(
        staticKey,
        signKey,
        channelName,
        unixTs,
        randomInt)
    ret = str(signature) + \
        staticKey + \
        '{0:0>10}'.format(unixTs) + \
        "%.8x" % (int(randomInt))
    return ret


def generateSignature5(
        servicetype,
        appID,
        appCertificate,
        channelName,
        unixTs,
        randomInt,
        uid,
        expiredTs,
        extra,
):
    randomInt = ctypes.c_uint(randomInt).value
    content = packUint16(servicetype) \
        + packString(appID.decode('hex'))\
        + packUint32(unixTs)  \
        + packUint32(randomInt)  \
        + packString(channelName) \
        + packUint32(uid)\
        + packUint32(expiredTs)\
        + packMap(extra)
    signature = hmac.new(
        appCertificate.decode('hex'),
        content, sha1).hexdigest()
    return signature.upper()


def generateSignature4(
        staticKey,
        signKey,
        channelName,
        unixTs,
        randomInt,
        uid,
        expiredTs,
        servicetype):
    key = "\x00" * (32 - len(staticKey)) + staticKey
    content = servicetype + key +\
        '{:0>10}'.format(unixTs) + \
        "%.8x" % (int(randomInt)) + \
        str(channelName) +\
        '{:0>10}'.format(uid) + \
        '{:0>10}'.format(expiredTs)
    signature = hmac.new(signKey, content, sha1).hexdigest()
    return signature


def parseToken4(token):
    signature = token[3:43]
    vendorKey = token[43:75]
    unixTs = int(token[75:85])
    randomInt = int("0x%s" % (token[85:93]), 16)
    expiredTs = int(token[93:103])
    logger.info(
        "vendorKey %s, signature %s, unixTs %s, randomInt %s, expiredTs %s",
        vendorKey,
        signature,
        unixTs,
        randomInt,
        expiredTs)
    return (signature, unixTs, randomInt, expiredTs)


def parseToken5(token):
    key = base64.b64decode(token[3:])
    size = 2
    signature_len = unpackUint16(key[size:size + 2])[0]
    size += 2
    signature = key[size:size + signature_len]
    size += signature_len
    appid_len = unpackUint16(key[size: size + 2])[0]
    size += 2
    vendorKey = key[size:size + appid_len].encode('hex')
    size += appid_len
    unixTs = unpackUint32(key[size:size + 4])[0]
    size += 4
    randomInt = unpackUint32(key[size:size + 4])[0]
    size += 4
    expiredTs = unpackUint32(key[size:size + 4])[0]
    logger.info(
        "vendorKey %s, signature %s, unixTs %s, randomInt %s, expiredTs %s",
        vendorKey,
        signature,
        unixTs,
        randomInt,
        expiredTs)
    return (signature, unixTs, randomInt, expiredTs)


def verify_ticket(channelName, uid, key):
    uid = ctypes.c_uint(uid).value
    version = str(key[:3])
    now = int(time.time())
    if version == "004":
        # if unixTs + TIMEOUT < now:
        # raise Exception(
        # "unixTs expired now : %d unixTs: %d" %
        # (now, unixTs))
        (signature, unixTs, randomInt, expiredTs) = parseToken4(key)
        gen = generateSignature4(
            VENDORKEY,
            VENDORSIGN,
            channelName,
            unixTs,
            randomInt,
            uid,
            expiredTs, "ARS")
        if gen != signature:
            raise Exception(
                "dynamic key 4 token diff expected:%s acctually:%s" %
                (gen, signature))
    elif version == "005":
        (signature, unixTs, randomInt, expiredTs) = parseToken5(key)
        logger.info(
            "signature, unixTs, randomInt, expiredTs cname uid:%s %s %s %s %s %s",
            signature,
            unixTs,
            randomInt,
            expiredTs,
            channelName,
            uid)
        gen = generateSignature5(
            2,
            VENDORKEY,
            VENDORSIGN,
            channelName,
            unixTs,
            randomInt,
            uid,
            expiredTs, {})
        if gen != signature:
            raise Exception(
                "dynamic key 5 token diff expected:%s acctually:%s" %
                (gen, signature))

    else:
        raise Exception("version error %s" % version)


class Process(object):
    def __init__(self):
        self.process = None

class DataHub(object):

    def __init__(self):
        self.processes = {}
        self.lock = locks.Lock()

    def clear_dead_process(self):
        self.lock.acquire()
        for cname, p in self.processes.items():
            if  not p.process :
                del self.processes[cname]
            if p.process and p.process.poll() is not None:
                logger.info("clear dead process %s" % (id(p)))
                p.process.wait()
                del self.processes[cname]
        self.lock.release()

    def getProcess(self, cname):
        self.lock.acquire()
        if cname not in self.processes:
            self.processes[cname]=Process()
        self.lock.release()
        return self.processes[cname]


dataHub = DataHub()


def getPid(channelName):
    for pid in os.listdir('/proc'):
        if not pid.isdigit():
            continue
        try:
            with open('/proc/{}/cmdline'.format(pid), mode='rb') as fd:
                content = fd.read().decode().split('\x00')
        except Exception:
            continue
        content = [str(x) for x in content]
        if channelName in content and "./agora_recording.exe" in content:
            return pid
    return None


def start(channelName, encryptType, secret):
    p = dataHub.getProcess(channelName)
    logger.info("start process %s channel %s encrypt:[%s ,%s]" %
                (id(p), channelName, encryptType, secret))
    needDecrypt = ''
    if encryptType != '' and secret != "":
        needDecrypt = "--encrypt_mode %s --encrypt_key %s" % (
            encryptType, secret)
    ret = 0
    if p.process and p.process.poll() is None:
        ret = 1
    else:
        dkey = generateDynamicKey(
            VENDORKEY, VENDORSIGN, channelName, int(time.time()), 0)
        cmd = "AGORA_FILE_ROOT=%s LD_LIBRARY_PATH=. ./agora_recording.exe --appid %s --key %s "\
            " --name '%s' --uid %s %s >> ars.log 2>&1" % \
            (AGORA_FILE_ROOT, VENDORKEY, dkey, channelName, int(time.time() * 1000), needDecrypt)
        logger.info("%s" % cmd)
        p.process = subprocess.Popen(cmd, shell=True)
        if p.process.poll() is None:
            ret = 1
        else:
            ret = 0
    return ret


def stop(channelName):
    p = dataHub.getProcess(channelName)
    logger.info("stop process %s channel %s" % (id(p), channelName))
    if p.process and p.process.poll() is None:
        p.process.terminate()
        p.process.wait()
        pid = getPid(channelName)
        if pid:
            os.kill(int(pid), signal.SIGTERM)
            logger.info("stoped agora_recording.exe pid:%s " % (pid))


def status(channelName):
    p = dataHub.getProcess(channelName)
    ret = 0
    if p and p.process and p.process.poll() is None:
        ret = 1
    else:
        ret = 0
    return ret


def decryptSecret(enType, enSecret):
    type, secret = "", ""
    try:
        if enType != "" and enSecret != "":
            rsa_pri = RSA.load_key('private_key.pem')
            secret = rsa_pri.private_decrypt(
                enSecret.decode('base64'), RSA.pkcs1_padding)
            type = rsa_pri.private_decrypt(
                enType.decode('base64'), RSA.pkcs1_padding)
    except:
        traceback.print_exc()
    return type, secret


class MainHandler(tornado.web.RequestHandler):

    def initialize(self):
        self.set_header("Access-Control-Allow-Origin", "*")

    def get(self, url):
        ret = ''
        startTs = time.time()
        logger.info("ON %s %s" % (url, self.request))
        try:
            paths = [str(i) for i in url.split("/", 3)]
            path = paths[0]
            channelName = paths[1]
            enType = self.get_argument('type', '')
            enSecret = self.get_argument('secret', '')
            if path == "start":
                uid = int(paths[2])
                key = paths[3]
                verify_ticket(channelName, uid, key)
                t, s = decryptSecret(enType, enSecret)
                ret = start(channelName, t, s)
            elif path == "stop":
                uid = int(paths[2])
                key = paths[3]
                verify_ticket(channelName, uid, key)
                ret = stop(channelName)
            elif path == "status":
                ret = status(channelName)
            else:
                raise Exception("path error:%s" % path)
            self.set_status(200)
        except:
            self.set_status(500)
            traceback.print_exc()
        finally:
            logger.info("request %s finish , return %s,  takes %d ms" %
                        (url, ret, int((time.time() - startTs) * 1000)))
            self.finish(str(ret))

VENDORKEY = os.environ.get('VENDORKEY', '')
VENDORSIGN = os.environ.get('VENDORSIGN', 'agora_vendor_recording_signature')
PORT = os.environ.get('PORT', 8001)
AGORA_FILE_ROOT = os.environ.get('AGORA_FILE_ROOT', "./")


def register():
    http_client = HTTPClient()
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
    public_key = ''
    with open("public_key.pem", "r") as f1:
        public_key = f1.read()
    now = int(time.time())
    msg = {
        "publicKey": public_key,
        "addr": "",
        "unixTs": now,
        "randomInt": now,
        "key": generateDynamicKey(
            VENDORKEY,
            VENDORSIGN,
            "register",
            now,
            now)}
    body = json.dumps(msg)
    try:
        # url = "http://127.0.0.1:8005/agora/recording/register"
        url = "https://recording.agora.io/agora/recording/register"
        res = http_client.fetch(url, method='POST', headers=headers, body=body)
        ret = str(res.body.decode('utf-8'))
        logger.info("register res:%s" % (ret))
    except:
        traceback.print_exc()
        ret = 'fail'
    return ret

if __name__ == "__main__":
    application = tornado.web.Application([
        (r"/agora/recording/(.*)", MainHandler),
    ])
    application.listen(PORT)
    ret = register()
    if ret != "succ":
        logger.info("register fail")
    else:
        tornado.ioloop.PeriodicCallback(
            dataHub.clear_dead_process, 30 * 1000).start()
        tornado.ioloop.IOLoop.instance().start()

