#!/bin/bash
cd `dirname $0`
if [[ $# <  4 ]] ; then 
    echo "USAGE: sudo ./deploy.sh --key=vendorKey --sign=signature --port=port --path=absolut_path" 
    echo "deploy failed"
    exit 1; 
fi
export LC_ALL=en_US.UTF-8
export LANG=en_US.UTF-8

init_env(){
    echo "init_env"
    if [ "$(which python2.7)"  == "" ]; then
        apt-get update  2>&1 >>ars.log
        apt-get install -y python2.7 2>&1 >>ars.log
    fi
    if [ "`which pip`" == '' ]; then
        echo "install pip"
        apt-get update 2>&1 >>ars.log
        apt-get install -y python-pip 2>&1 >>ars.log
        #python2.7 get-pip.py>>ars.log
    fi
    if [ "`python2.7 -c 'import tornado' 2>&1`" != '' ]; then
        echo "install tornado"
        apt-get update   2>&1 >>ars.log
        apt-get install libssl1.0.0 2>&1 >>ars.log
        apt-get install -y libssl-dev 2>&1 >>ars.log
        pip install tornado 2>&1 >>ars.log
    fi
    if [ "`python2.7 -c 'import M2Crypto' 2>&1`" != '' ]; then
        echo "install M2Crypto"
        apt-get update 2>&1 >>ars.log
        apt-get install -y python-m2crypto 2>&1 >>ars.log
    fi
    if [ ! -f private_key.pem ];then
        echo "generate rsa public and private key"
        sudo -u $LOGNAME openssl genpkey -algorithm RSA -out private_key.pem -pkeyopt rsa_keygen_bits:1024  2>&1 >>ars.log 
        sudo -u $LOGNAME openssl rsa -pubout -in private_key.pem -out public_key.pem  2>&1 >>ars.log
    fi
    if [[ ! -f /usr/local/bin/ffmpeg || "$(md5sum /usr/local/bin/ffmpeg | awk '{print $1}')" != "$(md5sum ./ffmpeg | awk '{print $1}')" ]]; then
        chmod a+x ffmpeg 
        cp ffmpeg /usr/local/bin/ -fv
    fi
    chmod a+x ./agora_recording.exe
}

check_env(){
    echo "check_env"
    if [ "`python2.7 -c 'import tornado' 2>&1`" != '' ]; then
        echo "python not install tornado"
        echo "deploy failed"
        exit 
    fi
    if [ "`python2.7 -c 'import M2Crypto' 2>&1`" != '' ]; then
        echo "python not install M2Crypto"
        echo "deploy failed"
        exit 
    fi
    if [ ! -f private_key.pem ];then
        echo "not find private_key.pem "
        echo "deploy failed"
        exit 
    fi
    if [ ! -f public_key.pem ];then
        echo "not find public_key.pem "
        echo "deploy failed"
        exit 
    fi
    if [[ "$AGORA_FILE_ROOT" == "" || "$VENDORKEY" == "" || "$VENDORSIGN" == ""  || "$PORT" == "" ]]; then
        echo "argument error"
        echo "deploy failed"
        exit 
    fi
    if [ ! -x agora_recording.exe ]; then
        echo "agora_recording.exe has no permission'"
        echo "deploy failed"
        exit
    fi
    if [ "$(uname -a | grep Ubuntu)"  == "" ]; then
        echo "os not ubuntu"
        echo "deploy failed"
        exit
    fi
    if [ "$(which python2.7)"  == "" ]; then
        echo "python not 2.7"
        echo "deploy failed"
        exit
    fi
    ret=$(python2.7 -c '#!/usr/bin/python
#encoding=utf-8
from M2Crypto import RSA
msg = "aaaa-aaaa"
rsa_pub = RSA.load_pub_key("public_key.pem")
ctxt = rsa_pub.public_encrypt(msg, RSA.pkcs1_padding)
rsa_pri = RSA.load_key("private_key.pem")
txt = rsa_pri.private_decrypt(ctxt, RSA.pkcs1_padding)
if msg == txt:
    print "ok"
')
    if [[ "$ret" != "ok" ]]; then
        echo "check public_key and private_key error"
        echo "deploy failed"
        exit
    fi
}

parse_argv(){
    for i in "$@"
    do
        case $i in
            --key*)
                VENDORKEY="${i#*=}"
                shift # past argument
                ;;
            --sign*)
                VENDORSIGN="${i#*=}"
                shift # past argument
                ;;
            --port*)
                PORT="${i#*=}"
                shift # past argument
                ;;
            --addr*)
                ADDR="${i#*=}"
                shift # past argument
                ;;
            --path*)
                AGORA_FILE_ROOT="${i#*=}"
                shift # past argument
                ;;
        esac
    done
}

AGORA_FILE_ROOT=""
VENDORKEY=""
VENDORSIGN=''
IP=''
PORT=''

>ars.log
init_env
parse_argv $@
check_env

#timer to transcode
echo '#!/bin/bash
sudo python2.7 '"`pwd`/agora_transcode_mp4.py $AGORA_FILE_ROOT 2>&1 >>$AGORA_FILE_ROOT/agora_hourly.log" | tee /etc/cron.hourly/agora_hourly 
chmod 777 /etc/cron.hourly/agora_hourly

#start service
pid=`netstat -lnp  | grep $PORT | grep -v grep | awk '{print $7}' | awk -F/ '{print $1}'`
if [ "$pid" != '' ]; then
    kill $pid
fi
sudo -u $LOGNAME AGORA_FILE_ROOT=$AGORA_FILE_ROOT VENDORKEY=$VENDORKEY VENDORSIGN=$VENDORSIGN PORT=$PORT ADDR=$ADDR nohup python2.7 agora_recording_service.py  >> ars.log 2>&1 &
sleep 1
#check if service is running
for x in `seq 0 5`
do
    sleep 1
    pid=`netstat -lnp  | grep $PORT | grep -v grep | awk '{print $7}' | awk -F/ '{print $1}'`
    if [ "$pid" != "" ] ; then
        for x in `seq 0 10`
        do
            sleep 1
            if [ "`cat ars.log | grep 'register res:succ'`" != "" ] ; then
                echo "deploy successfully"
                exit
            fi
        done
    fi
done

echo "deploy failed"
